#include <formatio.h>
#include <utility.h>
#include <ansi_c.h>
#include <userint.h>
#include "toolbox.h"
#include "animate.h"


typedef enum
    {
    PARSE_START,
    PARSE_FIND_SETTING,
    PARSE_VALUE,
    PARSE_DONE
    } ParseState;

typedef enum
    {
    PARSE_VAL_FRAME_INDEX,
    PARSE_VAL_TOP,
    PARSE_VAL_LEFT,
    PARSE_VAL_HEIGHT,
    PARSE_VAL_WIDTH
    } ParseValue;


#define DEFAULT_FRAME_INTERVAL      0.2
#define STOP_ON_LAST_FRAME_DEFAULT  0
#define STOP_AT_PATH_END_DEFAULT    0
#define DEFAULT_ENABLED_STATE       1

#define ANIMATE_CTRL_TYPE           "Animation Ctrl"

typedef struct
    {
    int     timerCtrl;
    char    *path;
    char    *currPathPosition;
    int     stopOnLastFrame;
    int     stopAtPathEnd;
    int     restartingOnLastFrame;
    int     panel;          /* the panel the animation control resides on */
    int     ctrl;           /* the pict ring the animation control is based on */
    } *AnimateInfo;


static int  CVICALLBACK AnimateTimerCallback(int panel, int timerControl, int event, void *callbackData, int eventData1, int eventData2);
static int  IsPathValid(char *path);
static int  ParsePathEntry(char *pathPtr, char **nextEntry, int *top, int *left, int *height, int *width, int *frameIndex);
static int  CVICALLBACK AnimateCtrlCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2);
static int  CreateAnimateInfo(int panel, int ctrl,AnimateInfo *newAnimateInfo);
static void DiscardAnimateInfo(AnimateInfo animateInfo);
static void AdvanceAnimationControl(AnimateInfo animateInfo);
static int  IsPathAtEnd(char *pathPtr);


/*****************************************************************************************/

static int CreateAnimateInfo(int panel, int ctrl, AnimateInfo *newAnimateInfo)
{
    int         error = 0;
    AnimateInfo animateInfo = 0;

    nullChk( animateInfo = (AnimateInfo)calloc(sizeof(*animateInfo), 1));
    errChk( animateInfo->timerCtrl = NewCtrl(panel, CTRL_TIMER, 0, -100, -100));
    errChk( InstallCtrlCallback(panel, animateInfo->timerCtrl, AnimateTimerCallback, (void *)animateInfo));

    errChk( ChainCtrlCallback(panel, ctrl, AnimateCtrlCallback, (void *)animateInfo, ANIMATE_CTRL_TYPE));

    animateInfo->panel = panel;
    animateInfo->ctrl = ctrl;

    if (newAnimateInfo)
        *newAnimateInfo = animateInfo;

Error:
    if (error < 0)
        {
        if (newAnimateInfo)
            *newAnimateInfo = 0;
        return error;
        }
    else
        return error;
}

/*****************************************************************************************/

static void DiscardAnimateInfo(AnimateInfo animateInfo)
{
    if (animateInfo)
        {
        free(animateInfo->path);
        if (animateInfo->timerCtrl > 0)
            DiscardCtrl(animateInfo->panel, animateInfo->timerCtrl);

        if (animateInfo->panel && animateInfo->ctrl)
            UnchainCtrlCallback(animateInfo->panel, animateInfo->ctrl, ANIMATE_CTRL_TYPE);
        
        free(animateInfo);
        }
}

/*****************************************************************************************/

int CVIFUNC AnimateCtrl_Create(int panel, char *label, int top, int left)
{
    int pictRing;
    int error  = 0;

    errChk( pictRing = NewCtrl (panel, CTRL_PICTURE_RING, label, top, left));

    return AnimateCtrl_ConvertFromPictRing(panel, pictRing);

Error:
    return error;
}

/*****************************************************************************************/

int CVIFUNC AnimateCtrl_ConvertFromPictRing(int panel, int pictRingCtrl)
{
    int         error = 0;
    AnimateInfo animateInfo = 0;

    errChk( CreateAnimateInfo(panel, pictRingCtrl, &animateInfo));

        /* set control defaults */
    AnimateCtrl_SetAttribute(panel, pictRingCtrl, ATTR_ANIMATE_FRAME_INTERVAL, DEFAULT_FRAME_INTERVAL);
    AnimateCtrl_SetAttribute(panel, pictRingCtrl, ATTR_ANIMATE_STOP_ON_LAST_FRAME, STOP_ON_LAST_FRAME_DEFAULT);
    AnimateCtrl_SetAttribute(panel, pictRingCtrl, ATTR_ANIMATE_STOP_AT_PATH_END, STOP_AT_PATH_END_DEFAULT);
    AnimateCtrl_SetAttribute(panel, pictRingCtrl, ATTR_ANIMATE_ENABLED, DEFAULT_ENABLED_STATE);

    return pictRingCtrl;

Error:
    DiscardAnimateInfo(animateInfo);
    return error;
}

/*****************************************************************************************/

    /* returns 1 if the path is valid, or a negative UIL error code if not */
static int IsPathValid(char *path)
{
    if (!path)
        return 1;   /* a NULL path is valid */
    else
        {
        char    *pathPtr = path;
        int     parseResult;
            do
                {
                parseResult = ParsePathEntry(pathPtr, &pathPtr, 0, 0, 0, 0, 0);
                } while (parseResult >= 1);

            if (parseResult == 0)
                return 1;       /* reached end of path without problems */
            else
                return parseResult;
        }
}

/*****************************************************************************************/

    /*  Returns 0 if the path has ended, returns 1 if an entry was parsed, or
        it returns a negative UIL error code.
    */
static int ParsePathEntry(char *pathPtr, char **nextEntry, int *top, int *left, int *height, int *width, int *frameIndex)
{
    int         result = 1;     /* default is successful parse */
    ParseValue  whichValue;
    ParseState  state = PARSE_START;
    int         value;
    int         foundPlusEquals;
    int         reverseSign;

    while (state != PARSE_DONE)
        {
        pathPtr += strspn (pathPtr, " \t");  /* skip whitespace */

        switch (state)
            {
            case PARSE_START:
                if (!*pathPtr)
                    {
                    state = PARSE_DONE;
                    result = 0;         /* no item found */
                    }
                else
                if (*pathPtr != '(')
                    {
                    state = PARSE_DONE;
                    result = UIEValueIsInvalidOrOutOfRange;
                    }
                else
                    {
                    pathPtr += 1;
                    state = PARSE_FIND_SETTING;
                    }
                break;

            case PARSE_FIND_SETTING:
                if (!*pathPtr)
                    {
                    state = PARSE_DONE;
                    result = UIEValueIsInvalidOrOutOfRange;
                    }

                if (strncmp("F", pathPtr, 1) == 0 || strncmp("f", pathPtr, 1) == 0 )
                    {
                    whichValue = PARSE_VAL_FRAME_INDEX;
                    pathPtr += 1;
                    state = PARSE_VALUE;
                    }
                else
                if (strncmp("T", pathPtr, 1) == 0 || strncmp("t", pathPtr, 1) == 0 )
                    {
                    whichValue = PARSE_VAL_TOP;
                    pathPtr += 1;
                    state = PARSE_VALUE;
                    }
                else
                if (strncmp("L", pathPtr, 1) == 0 || strncmp("l", pathPtr, 1) == 0 )
                    {
                    whichValue = PARSE_VAL_LEFT;
                    pathPtr += 1;
                    state = PARSE_VALUE;
                    }
                else
                if (strncmp("H", pathPtr, 1) == 0 || strncmp("h", pathPtr, 1) == 0 )
                    {
                    whichValue = PARSE_VAL_HEIGHT;
                    pathPtr += 1;
                    state = PARSE_VALUE;
                    }
                else
                if (strncmp("W", pathPtr, 1) == 0 || strncmp("w", pathPtr, 1) == 0 )
                    {
                    whichValue = PARSE_VAL_WIDTH;
                    pathPtr += 1;
                    state = PARSE_VALUE;
                    }
                else
                if (*pathPtr == ')')
                    {
                    pathPtr++;
                    state = PARSE_DONE;
                    }
                else
                    {
                    state = PARSE_DONE;
                    result = UIEValueIsInvalidOrOutOfRange;
                    }
                break;

            case PARSE_VALUE:
                reverseSign = FALSE;

                if (!*pathPtr)
                    {
                    state = PARSE_DONE;
                    result = UIEValueIsInvalidOrOutOfRange;
                    }

                pathPtr = pathPtr + strspn (pathPtr, " \t");  /* skip whitespace */

                if (*pathPtr == '+' && *(pathPtr + 1) == '=')
                    {
                    foundPlusEquals = TRUE;
                    pathPtr += 1;   /* skip '+' */
                    }
                else
                if (*pathPtr == '-' && *(pathPtr + 1) == '=')
                    {
                    foundPlusEquals = TRUE;
                    reverseSign = TRUE;
                    pathPtr += 1;   /* skip '-' */
                    }
                else
                    foundPlusEquals = FALSE;

                pathPtr = pathPtr + strspn (pathPtr, "= \t");  /* skip equal signs and whitespace */

                if (strspn (pathPtr + strspn(pathPtr, "+-"), "0123456789") == 0)
                    {
                    state = PARSE_DONE;
                    result = UIEValueIsInvalidOrOutOfRange;   /* no number present */
                    }

                value = atoi(pathPtr);

                if (reverseSign)
                    value = -value;

                pathPtr += strspn(pathPtr, "+-0123456789, \t"); /* skip the number and any commas or white space */

                switch (whichValue)
                    {
                    case PARSE_VAL_FRAME_INDEX:
                        if  (frameIndex)
                            *frameIndex = foundPlusEquals ? *frameIndex + value : value;
                        break;
                    case PARSE_VAL_TOP:
                        if (top)
                            *top = foundPlusEquals ? *top + value : value;
                        break;
                    case PARSE_VAL_LEFT:
                        if (left)
                            *left = foundPlusEquals ? *left + value : value;
                         break;
                    case PARSE_VAL_HEIGHT:
                        if (height)
                            *height = foundPlusEquals ? *height + value : value;
                        break;
                    case PARSE_VAL_WIDTH:
                        if (width)
                            *width = foundPlusEquals ? *width + value : value;
                        break;
                    }

                state = PARSE_FIND_SETTING;         /* find next setting */
                break;
            } /* end switch */
        }   /* end while */

    if (nextEntry)
        *nextEntry = pathPtr;

    return result;
}

/*****************************************************************************************/

    /*  Returns 0 if the path has ended, returns 1 otherwise. */
static int IsPathAtEnd(char *pathPtr)
{
    int atEnd = TRUE;

    if (pathPtr)
        {
        pathPtr += strspn (pathPtr, " \t");  /* skip whitespace */
        if (*pathPtr)
            atEnd = FALSE;
        }

    return atEnd;
}

/*****************************************************************************************/

int CVIFUNC_C AnimateCtrl_SetAttribute(int panel, int animateCtrl, int attribute, ...)
{
    int         error = 0;
    va_list     parmInfo;
    AnimateInfo animateInfo = 0;

    errChk( GetChainedCallbackData(panel, animateCtrl, ANIMATE_CTRL_TYPE, (void **)&animateInfo));

    va_start(parmInfo, attribute);

    switch (attribute)
        {
        case ATTR_ANIMATE_FRAME_INTERVAL:
            errChk( SetCtrlAttribute(panel, animateInfo->timerCtrl, ATTR_INTERVAL, va_arg(parmInfo, double)));
            break;
        case ATTR_ANIMATE_STOP_ON_LAST_FRAME:
            animateInfo->stopOnLastFrame = va_arg(parmInfo, int);
            break;
        case ATTR_ANIMATE_STOP_AT_PATH_END:
            animateInfo->stopAtPathEnd = va_arg(parmInfo, int);
            break;
        case ATTR_ANIMATE_ENABLED:
            {
            int wasEnabled;
            int isEnabled = va_arg(parmInfo, int);
            int frameIndex, numFrames;

            errChk( GetCtrlAttribute(panel, animateInfo->timerCtrl, ATTR_ENABLED, &wasEnabled));
            errChk( SetCtrlAttribute(panel, animateInfo->timerCtrl, ATTR_ENABLED, isEnabled));

            if (wasEnabled && !isEnabled)
                {
                errChk( GetCtrlIndex(panel, animateCtrl, &frameIndex));
                errChk( GetNumListItems(panel, animateCtrl, &numFrames));

                if (frameIndex + 1 >= numFrames)
                    animateInfo->restartingOnLastFrame = TRUE;
                }

            }
            break;
        case ATTR_ANIMATE_CURRENT_FRAME:
            errChk( SetCtrlIndex(panel, animateCtrl, va_arg(parmInfo, int)));
            break;
        case ATTR_ANIMATE_NUM_FRAMES:
            errChk( UIEAttributeNotSettable);
            break;
        case ATTR_ANIMATE_PATH:
            {
            char    *newPath;
            char    *newPathBuf;
            int     pathLength;

                newPath = (char *)va_arg(parmInfo, char *);

                errChk( IsPathValid(newPath));

                if (newPath)
                    {
                    pathLength = strlen(newPath);
                    nullChk( newPathBuf = (char *)malloc(pathLength+1));
                    strcpy(newPathBuf, newPath);
                    }
                else
                    newPathBuf = 0;

                free(animateInfo->path);
                animateInfo->path = newPathBuf;
                animateInfo->currPathPosition = animateInfo->path;
            }
            break;
        case ATTR_ANIMATE_PATH_TEXT_LENGTH:
            errChk( UIEAttributeNotSettable);
            break;
        default:
            errChk( UIEInvalidAttribute);
            break;
        }

    va_end(parmInfo);

Error:
    return error;
}

/*****************************************************************************************/

int CVIFUNC AnimateCtrl_GetAttribute(int panel, int animateCtrl, int attribute, void *value)
{
    int         error = 0;
    AnimateInfo animateInfo = 0;

    if (!value)
        errChk( UIENullPointerPassed);

    errChk( GetChainedCallbackData(panel, animateCtrl, ANIMATE_CTRL_TYPE, (void **)&animateInfo));

    switch (attribute)
        {
        case ATTR_ANIMATE_FRAME_INTERVAL:
            errChk( GetCtrlAttribute(panel, animateInfo->timerCtrl, ATTR_INTERVAL, value));
            break;
        case ATTR_ANIMATE_STOP_ON_LAST_FRAME:
            *(int *)value = animateInfo->stopOnLastFrame;
            break;
        case ATTR_ANIMATE_STOP_AT_PATH_END:
            *(int *)value = animateInfo->stopAtPathEnd;
            break;
        case ATTR_ANIMATE_ENABLED:
            errChk( GetCtrlAttribute(panel, animateInfo->timerCtrl, ATTR_ENABLED, value));
            break;
        case ATTR_ANIMATE_CURRENT_FRAME:
            errChk( GetCtrlIndex(panel, animateCtrl, value));
            break;
        case ATTR_ANIMATE_NUM_FRAMES:
            errChk( GetNumListItems(panel, animateCtrl, value));
            break;
        case ATTR_ANIMATE_PATH:
            strcpy(value, animateInfo->path);
            break;
        case ATTR_ANIMATE_PATH_TEXT_LENGTH:
            if (animateInfo->path)
                *(int *)value = strlen(animateInfo->path);
            else
                *(int *)value = 0;
            break;
        default:
            errChk( UIEInvalidAttribute);
            break;
        }

Error:
    return error;
}

/*****************************************************************************************/

static int  CVICALLBACK AnimateTimerCallback(int panel, int timerControl, int event, void *callbackData, int eventData1, int eventData2)
{
    AnimateInfo animateInfo = (AnimateInfo)callbackData;

    UNUSED(panel);
    UNUSED(timerControl);
    UNUSED(eventData1);
    UNUSED(eventData2);

    switch (event)
        {
        case EVENT_TIMER_TICK:
            AdvanceAnimationControl(animateInfo);
            break;
        case EVENT_DISCARD:
            animateInfo->timerCtrl = 0; /* if the panel is discarded, the timer might be discarded before the Animation Control, therefore, let the animation control know it has been discarded */
            break;
        }

    return 0;
}

/*****************************************************************************************/

static void  AdvanceAnimationControl(AnimateInfo animateInfo)
{
    int         numFrames;
    int         currentFrame;
    int         newFrame;
    int         top, left, height, width, newTop, newLeft, newHeight, newWidth;
    int         onLastFrame;
    int         parseResult;
    int         redrawNeeded = 0;
    int         userSaysToAbort;
    int         panel = animateInfo->panel;
    int         animationCtrl = animateInfo->ctrl;

    AnimateCtrl_GetAttribute(panel, animationCtrl, ATTR_ANIMATE_NUM_FRAMES, &numFrames);
    if (numFrames <= 0)
        return;

    AnimateCtrl_GetAttribute(panel, animationCtrl, ATTR_ANIMATE_CURRENT_FRAME, &currentFrame);

    newFrame = currentFrame + 1;
    onLastFrame = currentFrame + 1 >= numFrames;

    if (onLastFrame)
        {
        if (animateInfo->stopOnLastFrame && !animateInfo->restartingOnLastFrame)
            {
            AnimateCtrl_SetAttribute(panel, animationCtrl, ATTR_ANIMATE_ENABLED, 0);
            return;
            }
        else
            newFrame = 0;
        }

    animateInfo->restartingOnLastFrame = 0;

    if (animateInfo->stopAtPathEnd && animateInfo->path && animateInfo->currPathPosition)
        if (IsPathAtEnd(animateInfo->currPathPosition))
            {
            animateInfo->currPathPosition = animateInfo->path;
            AnimateCtrl_SetAttribute(panel, animationCtrl, ATTR_ANIMATE_ENABLED, 0);
            return;
            }

    if (CallCtrlCallback(panel, animationCtrl, EVENT_ANIMATE_PRE_FRAME, currentFrame, newFrame, &userSaysToAbort) && userSaysToAbort)
        return;

    if (animateInfo->path && animateInfo->currPathPosition)
        {
        GetCtrlAttribute(panel, animationCtrl, ATTR_TOP, &top);
        GetCtrlAttribute(panel, animationCtrl, ATTR_LEFT, &left);
        GetCtrlAttribute(panel, animationCtrl, ATTR_WIDTH, &width);
        GetCtrlAttribute(panel, animationCtrl, ATTR_HEIGHT, &height);

        newLeft = left;
        newTop = top;
        newHeight = height;
        newWidth = width;

        parseResult = ParsePathEntry(animateInfo->currPathPosition, &animateInfo->currPathPosition,
                                    &newTop, &newLeft,&newHeight, &newWidth, &newFrame);

        if (parseResult == 0)   /* if at end of the path then try again from start of path */
            {
            animateInfo->currPathPosition = animateInfo->path;
            parseResult = ParsePathEntry(animateInfo->currPathPosition, &animateInfo->currPathPosition,
                                        &newTop, &newLeft, &newHeight, &newWidth, &newFrame);
            }

        if (newFrame < 0)       /* make sure the specified frame index is reasonable */
            newFrame = 0;
        else
        if (newFrame >= numFrames)
            newFrame = numFrames - 1;

        if (newTop != top)
            {
            SetCtrlAttribute(panel, animationCtrl, ATTR_TOP, newTop);
            redrawNeeded = 1;
            }

        if (newHeight != height || newWidth != width)
            SetCtrlAttribute (panel, animationCtrl, ATTR_FIT_MODE, VAL_SIZE_TO_PICTURE);

        if (newLeft != left)
            {
            SetCtrlAttribute(panel, animationCtrl, ATTR_LEFT, newLeft);
            redrawNeeded = 1;
            }

        if (newWidth != width)
            {
            SetCtrlAttribute(panel, animationCtrl, ATTR_WIDTH, newWidth);
            redrawNeeded = 1;
            }

        if (newHeight != height)
            {
            SetCtrlAttribute(panel, animationCtrl, ATTR_HEIGHT, newHeight);
            redrawNeeded = 1;
            }
        }

    AnimateCtrl_SetAttribute(panel, animationCtrl, ATTR_ANIMATE_CURRENT_FRAME, newFrame);
    if (redrawNeeded)
        ProcessDrawEvents();

    CallCtrlCallback(panel, animationCtrl, EVENT_ANIMATE_POST_FRAME, currentFrame, newFrame, 0);
}

/*****************************************************************************************/

static int  CVICALLBACK AnimateCtrlCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    AnimateInfo animateInfo =  (AnimateInfo)callbackData;

    UNUSED(panel);
    UNUSED(control);
    UNUSED(eventData1);
    UNUSED(eventData2);

    switch (event)
        {
        case EVENT_DISCARD:
            DiscardAnimateInfo(animateInfo);
            break;
        }

    return 0;
}
