/*============================================================================*/
/*                        L a b W i n d o w s / C V I                         */
/*----------------------------------------------------------------------------*/
/*    Copyright (c) National Instruments 1987-1996.  All Rights Reserved.     */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/* Title:       daq_num.h                                                     */
/* Purpose:     implements analog input/output numeric controls               */
/*                                                                            */
/*============================================================================*/

#ifndef DAQ_NUMERIC_HEADER
#define DAQ_NUMERIC_HEADER

#ifdef _CVI_
#pragma EnableLibraryRuntimeChecking
#endif

#include "userint.h"
#include "easyio.h"
#include "daqctrl.h"

#ifdef __cplusplus
    extern "C" {
#endif

enum DaqNumAttribute
    {
        /* base class attributes */
    ATTR_DAQ_NUMERIC_DEVICE                     = ATTR_DAQ_CTRL_DEVICE,
    ATTR_DAQ_NUMERIC_BOARD                      = ATTR_DAQ_CTRL_DEVICE, /* for compatability with the beta release */
    ATTR_DAQ_NUMERIC_CHANNEL_STRING             = ATTR_DAQ_CTRL_CHANNEL_STRING,
    ATTR_DAQ_NUMERIC_CHANNEL_STRING_COPY        = ATTR_DAQ_CTRL_CHANNEL_STRING_COPY,
    ATTR_DAQ_NUMERIC_CHANNEL_STRING_LENGTH      = ATTR_DAQ_CTRL_CHANNEL_STRING_LENGTH,
    ATTR_DAQ_NUMERIC_HIGH_LIMIT                 = ATTR_DAQ_CTRL_HIGH_LIMIT,
    ATTR_DAQ_NUMERIC_LOW_LIMIT                  = ATTR_DAQ_CTRL_LOW_LIMIT,
    ATTR_DAQ_NUMERIC_ENABLED                    = ATTR_DAQ_CTRL_ENABLED,
    ATTR_DAQ_NUMERIC_IO_DIRECTION               = ATTR_DAQ_CTRL_IO_DIRECTION,
    ATTR_DAQ_NUMERIC_VISIBLE                    = ATTR_DAQ_CTRL_VISIBLE,
    ATTR_DAQ_NUMERIC_ALLOW_RCLICK_EDIT          = ATTR_DAQ_CTRL_ALLOW_RCLICK_EDIT,
    ATTR_DAQ_NUMERIC_EDIT_DIALOG_TITLE          = ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE,
    ATTR_DAQ_NUMERIC_EDIT_DIALOG_TITLE_COPY     = ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE_COPY,
    ATTR_DAQ_NUMERIC_EDIT_DIALOG_TITLE_LENGTH   = ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE_LENGTH,
    ATTR_DAQ_NUMERIC_EDIT_DIALOG_DISPLAYED      = ATTR_DAQ_CTRL_EDIT_DIALOG_DISPLAYED,
    ATTR_DAQ_NUMERIC_EDIT_DIALOG_MODAL          = ATTR_DAQ_CTRL_EDIT_DIALOG_MODAL,
    ATTR_DAQ_NUMERIC_TIE_CTRL_RANGE_TO_LIMITS   = ATTR_DAQ_CTRL_TIE_CTRL_RANGE_TO_LIMITS,
    ATTR_DAQ_NUMERIC_VERSION                    = ATTR_DAQ_CTRL_VERSION,
    ATTR_DAQ_NUMERIC_ERROR_NOTIFICATION         = ATTR_DAQ_CTRL_ERROR_NOTIFICATION,

        /* derived class attributes */
    ATTR_DAQ_NUMERIC_CTRL_VAL                   = 3100,
    ATTR_DAQ_NUMERIC_SCAN_FREQUENCY             = 3101
    };


#define DAQ_NUMERIC_MIN_SCAN_FREQUENCY          1.0E-8
#define DAQ_NUMERIC_MAX_SCAN_FREQUENCY          100.0


    /*  If a daq numeric has a callback function and is in input mode, then
        the following event is sent to the callback each time the analog input
        channel is read.  eventData1 is a pointer to a double which holds the
        value of the voltage read, and eventData2 is the result code of the
        easyI/O operation which will be negative if an error occurred.
    */
#define EVENT_DAQ_NUMERIC_DATA_READY            4000


int CVIFUNC DAQ_Numeric_Create(int panel, int style, char *label, int ioDirection, int top, int left, char *configurationName, CtrlCallbackPtr callback, void *callbackData);
int CVIFUNC DAQ_Numeric_ConvertFromNumeric(int panel, int numeric, char *configurationName);
int CVIFUNC_C DAQ_Numeric_SetAttribute(int panel, int daqNumeric, int attribute, ...);
int CVIFUNC DAQ_Numeric_GetAttribute(int panel, int daqNumeric, int attribute, void *value);
int CVIFUNC DAQ_Numeric_EditPopup(int panel, int daqNumeric, char *dialogTitle, int modal);
int CVIFUNC DAQ_Numeric_SaveConfig(int panel, int daqNumeric, char *configurationName);
int CVIFUNC DAQ_Numeric_RecallConfig(int panel, int daqNumeric, char *configurationName);


/* This function is for use by the daq numeric user protection module (ckdaqnum.obj) */
int CVIFUNC DAQ_Numeric_SetAttributeFromParmInfo(int panel, int daqChart, int attribute, va_list parmInfo);

#ifdef __cplusplus
    }
#endif

#endif /* ifndef DAQ_NUMERIC_HEADER */
