/*============================================================================*/
/*                        L a b W i n d o w s / C V I                         */
/*----------------------------------------------------------------------------*/
/*    Copyright (c) National Instruments 1987-1996.  All Rights Reserved.     */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/* Title:       daqchart.h                                                    */
/* Purpose:     implements DAQ strip chart controls                           */
/*                                                                            */
/*============================================================================*/

#ifndef DAQ_CHART_CTRL_HEADER
#define DAQ_CHART_CTRL_HEADER

#ifdef _CVI_
#pragma EnableLibraryRuntimeChecking
#endif

#include <userint.h>    /* for data type defines and colors */
#include "easyio.h"
#include "daqctrl.h"
#include "datalog.h"

#ifdef __cplusplus
    extern "C" {
#endif


        /* base class attributes (from daqctrl.h) */
#define ATTR_DAQ_CHART_DEVICE                       ATTR_DAQ_CTRL_DEVICE
#define ATTR_DAQ_CHART_BOARD                        ATTR_DAQ_CTRL_DEVICE        /* for compatability with the beta release */
#define ATTR_DAQ_CHART_CHANNEL_STRING               ATTR_DAQ_CTRL_CHANNEL_STRING
#define ATTR_DAQ_CHART_CHANNEL_STRING_COPY          ATTR_DAQ_CTRL_CHANNEL_STRING_COPY
#define ATTR_DAQ_CHART_CHANNEL_STRING_LENGTH        ATTR_DAQ_CTRL_CHANNEL_STRING_LENGTH
#define ATTR_DAQ_CHART_HIGH_LIMIT                   ATTR_DAQ_CTRL_HIGH_LIMIT
#define ATTR_DAQ_CHART_LOW_LIMIT                    ATTR_DAQ_CTRL_LOW_LIMIT
#define ATTR_DAQ_CHART_REFRESH_FREQUENCY            ATTR_DAQ_CTRL_REFRESH_FREQUENCY
#define ATTR_DAQ_CHART_ENABLED                      ATTR_DAQ_CTRL_ENABLED
#define ATTR_DAQ_CHART_IO_DIRECTION                 ATTR_DAQ_CTRL_IO_DIRECTION
#define ATTR_DAQ_CHART_VISIBLE                      ATTR_DAQ_CTRL_VISIBLE
#define ATTR_DAQ_CHART_ALLOW_RCLICK_EDIT            ATTR_DAQ_CTRL_ALLOW_RCLICK_EDIT
#define ATTR_DAQ_CHART_EDIT_DIALOG_TITLE            ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE
#define ATTR_DAQ_CHART_EDIT_DIALOG_TITLE_COPY       ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE_COPY
#define ATTR_DAQ_CHART_EDIT_DIALOG_TITLE_LENGTH     ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE_LENGTH
#define ATTR_DAQ_CHART_EDIT_DIALOG_DISPLAYED        ATTR_DAQ_CTRL_EDIT_DIALOG_DISPLAYED
#define ATTR_DAQ_CHART_EDIT_DIALOG_MODAL            ATTR_DAQ_CTRL_EDIT_DIALOG_MODAL
#define ATTR_DAQ_CHART_TIE_CTRL_RANGE_TO_LIMITS     ATTR_DAQ_CTRL_TIE_CTRL_RANGE_TO_LIMITS
#define ATTR_DAQ_CHART_VERSION                      ATTR_DAQ_CTRL_VERSION
#define ATTR_DAQ_CHART_ERROR_NOTIFICATION           ATTR_DAQ_CTRL_ERROR_NOTIFICATION

        /* derived class attributes */
#define ATTR_DAQ_CHART_WARN_ON_HW_TIMING_RESTARTS   3200
#define ATTR_DAQ_CHART_USE_HARDWARE_TIMIMG          3201
#define ATTR_DAQ_CHART_SCAN_FREQUENCY               3202
#define ATTR_DAQ_CHART_NUM_CHANNELS                 3203
#define ATTR_DAQ_CHART_HISTORY_LENGTH               3204
#define ATTR_DAQ_CHART_HISTORY_BUFFER_SIZE          3205


        /* datalog attributes (from datalog.h.  Other datalog attributes in datalog.h may be passed, but they are undocumented and not useful for general applications) */
#define ATTR_DATALOG_LOGGING_ON                     3400
#define ATTR_DATALOG_LOGGING_PAUSED                 3401
#define ATTR_DATALOG_LOGGING_STATE                  3402    /* see DataLogState enum in datalog.h */
#define ATTR_DATALOG_DATA_LOG_FILE                  3403
#define ATTR_DATALOG_EVENT_LOG_FILE                 3406
#define ATTR_DATALOG_LOG_EVENTS                     3407
#define ATTR_DATALOG_LOG_FILE_FORMAT                3408    /* see DataLogFormat enum in datalog.h */
#define ATTR_DATALOG_START_LOGGING_ON_LOAD          3409
#define ATTR_DATALOG_PROMPT_ON_LOG_OVERWRITE        3410
#define ATTR_DATALOG_LOG_FIELD_WIDTH                3411
#define ATTR_DATALOG_WRITE_LOG_HEADER               3412
#define ATTR_DATALOG_COLUMN_HEADINGS                3413
#define ATTR_DATALOG_COLUMN_HEADINGS_COPY           3414
#define ATTR_DATALOG_COLUMN_HEADINGS_LENGTH         3415
#define ATTR_DATALOG_COMMENT                        3416
#define ATTR_DATALOG_COMMENT_COPY                   3417
#define ATTR_DATALOG_COMMENT_LENGTH                 3418
#define ATTR_DATALOG_LOG_MODE                       3419    /* see DataLogMode enum in datalog.h */
#define ATTR_DATALOG_NUM_SCANS_TO_LOG               3420
#define ATTR_DATALOG_NUM_SCANS_LOGGED               3421
#define ATTR_DATALOG_LOG_CONDITIONALLY              3425

    /*  Alarm Attributes: */
#define ATTR_ALARM_LIMIT                            3250
#define ATTR_ALARM_UPPER_LIMIT                      ATTR_ALARM_LIMIT
#define ATTR_ALARM_LOWER_LIMIT                      3251
#define ATTR_ALARM_DEADBAND                         3252
#define ATTR_ALARM_MODE                             3253
#define ATTR_ALARM_PLOT                             3254
#define ATTR_ALARM_TRACE_COLOR                      3255
#define ATTR_ALARM_NAME                             3256
#define ATTR_ALARM_CHANNEL                          3257


#define VAL_DAQ_CHART_REFRESH_VERY_FAST         25.0
#define VAL_DAQ_CHART_REFRESH_FAST              10.0
#define VAL_DAQ_CHART_REFRESH_NORMAL            5.0
#define VAL_DAQ_CHART_REFRESH_SLOW              2.0
#define VAL_DAQ_CHART_REFRESH_VERY_SLOW         0.5

#define DAQ_CHART_MAX_SCAN_FREQUENCY            ((double)10000.0)
#define DAQ_CHART_MIN_SCAN_FREQUENCY            ((double)0.00001)

#define DAQ_CHART_MAX_REFRESH_FREQUENCY         100.0
#define DAQ_CHART_MIN_REFRESH_FREQUENCY         1.0E-8

#define DAQ_CHART_MIN_REFRESH_INTERVAL          0.01

#define DAQ_CHART_MAX_ALARM_NAME_LENGTH         64


    /*  Events received by a DAQ Chart's callback function: */
#define EVENT_DAQ_CHART_DATA_READY              4100    /* (double *)eventData1 == scannedData, eventData2 == daqError code */
#define EVENT_DAQ_CHART_ALARM_ON                4101    /* (char *)eventData1 == alarmName, *(double *)eventData2 == value */
#define EVENT_DAQ_CHART_ALARM_OFF               4102    /* (char *)eventData1 == alarmName, *(double *)eventData2 == value */
#define EVENT_DAQ_CHART_HW_TIMED_SCAN_RESTARTED 4103    /* eventData1 == VAL_DAQ_CHART_HW_TIMED_SCAN_BUF_OVERRUN, or VAL_DAQ_CHART_HW_TIMED_TRANSFER_ABORTED */


    /*  Values for eventData1 when the DAQ Chart's callback function receives the
        EVENT_DAQ_CHART_HW_TIMED_SCAN_RESTARTED event.
    */
#define VAL_DAQ_CHART_HW_TIMED_SCAN_BUF_OVERRUN 1
#define VAL_DAQ_CHART_HW_TIMED_TRANSFER_ABORTED 2

    /*  Values for the mode parameter to DAQ_Chart_CreateAlarm(): */
#define VAL_ALARM_ABOVE_LIMIT       1
#define VAL_ALARM_BELOW_LIMIT       2
#define VAL_ALARM_IN_RANGE          3
#define VAL_ALARM_OUT_OF_RANGE      4

    /*  old names used during beta (obsolete, don't use): */
#define VAL_DAQ_CHART_ABOVE_LIMIT   VAL_ALARM_ABOVE_LIMIT
#define VAL_DAQ_CHART_BELOW_LIMIT   VAL_ALARM_BELOW_LIMIT
#define VAL_DAQ_CHART_IN_RANGE      VAL_ALARM_IN_RANGE
#define VAL_DAQ_CHART_OUT_OF_RANGE  VAL_ALARM_OUT_OF_RANGE


int CVIFUNC DAQ_Chart_Create(int panel, char *label, int top, int left, char *configurationName,  CtrlCallbackPtr callback, void *callbackData);
int CVIFUNC DAQ_Chart_ConvertFromStripChart(int panel, int stripChart, char *configurationName);
int CVIFUNC_C DAQ_Chart_SetAttribute(int panel, int daqChart, int attribute, ...);
int CVIFUNC DAQ_Chart_GetAttribute(int panel, int daqChart, int attribute, void *value);
int CVIFUNC DAQ_Chart_EditPopup(int panel, int daqChart, char *dialogTitle, int modal);
int CVIFUNC DAQ_Chart_GetHistory(int panel, int daqChart, int *numPoints, int *numScans,
        int *channelsPerScan, int groupDataByChannels, double samples[], double sampleTimes[], int remove);
int CVIFUNC DAQ_Chart_GetHistoryCopy(int panel, int daqChart, int *numPoints, int *numScans,
        int *channelsPerScan, int groupDataByChannels, double **samples, double **sampleTimes, int remove);
int CVIFUNC DAQ_Chart_PlotHistory(int panel, int daqChartCtrl, int graphPanel, int graph, int plotVsTime,
        int numScansToPlot, int plotAlarmLimits, int deleteCurrentPlots, int remove, int matchYAxisRange);
int CVIFUNC DAQ_Chart_CreateAlarm(int panel, int daqChart, char *alarmName, char *channel, double limit,
        double lowerLimit, double deadband, int mode, int plotTheLimit, int traceColor);
int CVIFUNC DAQ_Chart_DiscardAlarm(int panel, int daqChart, char *alarmName);
int CVIFUNC_C DAQ_Chart_SetAlarmAttribute(int panel, int chart, char *alarmName, int attribute, ...);
int CVIFUNC DAQ_Chart_GetAlarmAttribute(int panel, int chart, char *alarmName, int attribute, void *value);
int CVIFUNC DAQ_Chart_RecallConfig(int panel, int daqChart, char *configurationName);
int CVIFUNC DAQ_Chart_SaveConfig(int panel, int daqChart, char *configurationName);
int CVIFUNC DAQ_Chart_SortLogFile(char *logFilePath, int displayProgressDialog);



/* These functions are for use by the user protection module (ckdaqcht.obj) */
int CVIFUNC DAQ_Chart_SetAttributeFromParmInfo(int panel, int daqChart, int attribute, va_list parmInfo);
int CVIFUNC DAQ_Chart_SetAlarmAttributeFromParmInfo(int panel, int daqChart, char alarmName[], int attribute, va_list parmInfo);

#ifdef __cplusplus
    }
#endif

#endif /* DAQ_CHART_CTRL_HEADER */
